#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# ------------------------------------------------------- #
# Records
# Record access API.
# Specification: workspace:v1.0
# ------------------------------------------------------- #

from xata.api_request import ApiRequest
from xata.api_response import ApiResponse


class Records(ApiRequest):

    scope = "workspace"

    def transaction(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Execute a transaction on a branch

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/transaction#execute-a-transaction-on-a-branch
        Path: /db/{db_branch_name}/transaction
        Method: POST
        Response status codes:
        - 200: Returns the results of a successful transaction.
        - 400: Returns errors from a failed transaction.
        - 401: Authentication Error
        - 404: Example response
        - 429: Rate limit exceeded
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/transaction"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def insert(
        self, table_name: str, payload: dict, db_name: str = None, branch_name: str = None, columns: list = None
    ) -> ApiResponse:
        """
        Insert a new Record into the Table

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data#insert-record
        Path: /db/{db_branch_name}/tables/{table_name}/data
        Method: POST
        Response status codes:
        - 201: Record ID and metadata
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data"
        if columns is not None:
            url_path += "?columns=%s" % ",".join(columns)
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def get(
        self, table_name: str, record_id: str, db_name: str = None, branch_name: str = None, columns: list = None
    ) -> ApiResponse:
        """
        Retrieve record by ID

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data/record_id#get-record-by-id
        Path: /db/{db_branch_name}/tables/{table_name}/data/{record_id}
        Method: GET
        Response status codes:
        - 200: Table Record Reponse
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param record_id: str The Record name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data/{record_id}"
        if columns is not None:
            url_path += "?columns=%s" % ",".join(columns)
        return self.request("GET", url_path)

    def insert_with_id(
        self,
        table_name: str,
        record_id: str,
        payload: dict,
        db_name: str = None,
        branch_name: str = None,
        columns: list = None,
        create_only: bool = None,
        if_version: int = None,
    ) -> ApiResponse:
        """
        By default, IDs are auto-generated when data is insterted into Xata. Sending a request to this endpoint allows us to insert a record with a pre-existing ID, bypassing the default automatic ID generation.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data/record_id#insert-record-with-id
        Path: /db/{db_branch_name}/tables/{table_name}/data/{record_id}
        Method: PUT
        Response status codes:
        - 200: Record ID and metadata
        - 201: Record ID and metadata
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param record_id: str The Record name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters
        :param create_only: bool = None
        :param if_version: int = None

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data/{record_id}"
        query_params = []
        if columns is not None:
            query_params.append("columns=%s" % ",".join(columns))
        if create_only is not None:
            query_params.append(f"createOnly={create_only}")
        if if_version is not None:
            query_params.append(f"ifVersion={if_version}")
        if query_params:
            url_path += "?" + "&".join(query_params)
        headers = {"content-type": "application/json"}
        return self.request("PUT", url_path, headers, payload)

    def upsert(
        self,
        table_name: str,
        record_id: str,
        payload: dict,
        db_name: str = None,
        branch_name: str = None,
        columns: list = None,
        if_version: int = None,
    ) -> ApiResponse:
        """
        Upsert record with ID

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data/record_id#upsert-record-with-id
        Path: /db/{db_branch_name}/tables/{table_name}/data/{record_id}
        Method: POST
        Response status codes:
        - 200: Record ID and metadata
        - 201: Record ID and metadata
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param record_id: str The Record name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters
        :param if_version: int = None

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data/{record_id}"
        query_params = []
        if columns is not None:
            query_params.append("columns=%s" % ",".join(columns))
        if if_version is not None:
            query_params.append(f"ifVersion={if_version}")
        if query_params:
            url_path += "?" + "&".join(query_params)
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def delete(
        self, table_name: str, record_id: str, db_name: str = None, branch_name: str = None, columns: list = None
    ) -> ApiResponse:
        """
        Delete record from table

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data/record_id#delete-record-from-table
        Path: /db/{db_branch_name}/tables/{table_name}/data/{record_id}
        Method: DELETE
        Response status codes:
        - 200: Table Record Reponse
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param record_id: str The Record name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data/{record_id}"
        if columns is not None:
            url_path += "?columns=%s" % ",".join(columns)
        return self.request("DELETE", url_path)

    def update(
        self,
        table_name: str,
        record_id: str,
        payload: dict,
        db_name: str = None,
        branch_name: str = None,
        columns: list = None,
        if_version: int = None,
    ) -> ApiResponse:
        """
        Update record with ID

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/data/record_id#update-record-with-id
        Path: /db/{db_branch_name}/tables/{table_name}/data/{record_id}
        Method: PATCH
        Response status codes:
        - 200: Record ID and metadata
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param record_id: str The Record name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters
        :param if_version: int = None

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/data/{record_id}"
        query_params = []
        if columns is not None:
            query_params.append("columns=%s" % ",".join(columns))
        if if_version is not None:
            query_params.append(f"ifVersion={if_version}")
        if query_params:
            url_path += "?" + "&".join(query_params)
        headers = {"content-type": "application/json"}
        return self.request("PATCH", url_path, headers, payload)

    def bulk_insert(
        self, table_name: str, payload: dict, db_name: str = None, branch_name: str = None, columns: list = None
    ) -> ApiResponse:
        """
        Bulk insert records

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/bulk#bulk-insert-records
        Path: /db/{db_branch_name}/tables/{table_name}/bulk
        Method: POST
        Status: Experimental
        Response status codes:
        - 200: OK
        - 400: Response with multiple errors of the bulk execution
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param columns: list = None Column filters

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/bulk"
        if columns is not None:
            url_path += "?columns=%s" % ",".join(columns)
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)



#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# ------------------------------------------------------- #
# Branch
# Branch management.
# Specification: workspace:v1.0
# ------------------------------------------------------- #

from xata.api_request import ApiRequest
from xata.api_response import ApiResponse


class Branch(ApiRequest):

    scope = "workspace"

    def list(self, db_name: str) -> ApiResponse:
        """
        List all available Branches

        Reference: https://xata.io/docs/api-reference/dbs/db_name#list-branches
        Path: /dbs/{db_name}
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str The Database Name

        :returns ApiResponse
        """
        url_path = f"/dbs/{db_name}"
        return self.request("GET", url_path)

    def get_details(self, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Get branch schema and metadata

        Reference: https://xata.io/docs/api-reference/db/db_branch_name#get-branch-schema-and-metadata
        Path: /db/{db_branch_name}
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}"
        return self.request("GET", url_path)

    def create(self, payload: dict, db_name: str = None, branch_name: str = None, from_: str = None) -> ApiResponse:
        """
        Create Database branch

        Reference: https://xata.io/docs/api-reference/db/db_branch_name#create-database-branch
        Path: /db/{db_branch_name}
        Method: PUT
        Response status codes:
        - 201: Created
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 423: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.
        :param from_: str = None Name of source branch to branch the new schema from

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}"
        if from_ is not None:
            url_path += f"?from={from_}"
        headers = {"content-type": "application/json"}
        return self.request("PUT", url_path, headers, payload)

    def delete(self, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Delete the branch in the database and all its resources

        Reference: https://xata.io/docs/api-reference/db/db_branch_name#delete-database-branch
        Path: /db/{db_branch_name}
        Method: DELETE
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 409: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}"
        return self.request("DELETE", url_path)

    def get_metadata(self, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Get Branch Metadata

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/metadata#get-branch-metadata
        Path: /db/{db_branch_name}/metadata
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/metadata"
        return self.request("GET", url_path)

    def update_metadata(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Update the branch metadata

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/metadata#update-branch-metadata
        Path: /db/{db_branch_name}/metadata
        Method: PUT
        Response status codes:
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/metadata"
        headers = {"content-type": "application/json"}
        return self.request("PUT", url_path, headers, payload)

    def get_stats(self, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Get branch usage metrics.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/stats#branch-stats
        Path: /db/{db_branch_name}/stats
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Example response
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/stats"
        return self.request("GET", url_path)

    def get_git_branches_mapping(self, db_name: str) -> ApiResponse:
        """
        Lists all the git branches in the mapping, and their associated Xata branches.

        Example response:

        ```json
        {
          "mappings": [
              {
                "gitBranch": "main",
                "xataBranch": "main"
              },
              {
                "gitBranch": "gitBranch1",
                "xataBranch": "xataBranch1"
              }
              {
                "gitBranch": "xataBranch2",
                "xataBranch": "xataBranch2"
              }
          ]
        }
        ```

        Reference: https://xata.io/docs/api-reference/dbs/db_name/gitBranches#list-git-branches-mapping
        Path: /dbs/{db_name}/gitBranches
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str The Database Name

        :returns ApiResponse
        """
        url_path = f"/dbs/{db_name}/gitBranches"
        return self.request("GET", url_path)

    def add_git_branches_entry(self, db_name: str, payload: dict) -> ApiResponse:
        """
        Adds an entry to the mapping of git branches to Xata branches. The git branch and the Xata branch must be present in the body of the request. If the Xata branch doesn't exist, a 400 error is returned.

        If the git branch is already present in the mapping, the old entry is overwritten, and a warning message is included in the response. If the git branch is added and didn't exist before, the response code is 204. If the git branch existed and it was overwritten, the response code is 201.

        Example request:

        ```json
        // POST https://tutorial-ng7s8c.xata.sh/dbs/demo/gitBranches
        {
          "gitBranch": "fix/bug123",
          "xataBranch": "fix_bug"
        }
        ```

        Reference: https://xata.io/docs/api-reference/dbs/db_name/gitBranches#link-a-git-branch-to-a-xata-branch
        Path: /dbs/{db_name}/gitBranches
        Method: POST
        Response status codes:
        - 201: Operation was successful with warnings
        - 204: Operation was successful without warnings
        - 400: Bad Request
        - 401: Authentication Error
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str The Database Name
        :param payload: dict content

        :returns ApiResponse
        """
        url_path = f"/dbs/{db_name}/gitBranches"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def remove_git_branches_entry(self, db_name: str, git_branch: str) -> ApiResponse:
        """
        Removes an entry from the mapping of git branches to Xata branches. The name of the git branch must be passed as a query parameter. If the git branch is not found, the endpoint returns a 404 status code.

        Example request:

        ```json
        // DELETE https://tutorial-ng7s8c.xata.sh/dbs/demo/gitBranches?gitBranch=fix%2Fbug123
        ```

        Reference: https://xata.io/docs/api-reference/dbs/db_name/gitBranches#unlink-a-git-branch-to-a-xata-branch
        Path: /dbs/{db_name}/gitBranches
        Method: DELETE
        Response status codes:
        - 204: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: The git branch was not found in the mapping
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param db_name: str The Database Name
        :param git_branch: str The Git Branch to remove from the mapping

        :returns ApiResponse
        """
        url_path = f"/dbs/{db_name}/gitBranches"
        if git_branch is not None:
            url_path += f"?gitBranch={git_branch}"
        return self.request("DELETE", url_path)

    def resolve(self, db_name: str, git_branch: str = None, fallback_branch: str = None) -> ApiResponse:
        """
        In order to resolve the database branch, the following algorithm is used:
        * if the `gitBranch` was provided and is found in the [git branches mapping](/api-reference/dbs/db_name/gitBranches), the associated Xata branch is returned
        * else, if a Xata branch with the exact same name as `gitBranch` exists, return it
        * else, if `fallbackBranch` is provided and a branch with that name exists, return it
        * else, return the default branch of the DB (`main` or the first branch)

        Example call:

        ```json
        // GET https://tutorial-ng7s8c.xata.sh/dbs/demo/dbs/demo/resolveBranch?gitBranch=test&fallbackBranch=tsg
        ```

        Example response:

        ```json
        {
          "branch": "main",
          "reason": {
            "code": "DEFAULT_BRANCH",
            "message": "Default branch for this database (main)"
          }
        }
        ```

        Reference: https://xata.io/docs/api-reference/dbs/db_name/resolveBranch#resolve-a-git-branch-to-a-xata-branch
        Path: /dbs/{db_name}/resolveBranch
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param db_name: str The Database Name
        :param git_branch: str = None The Git Branch
        :param fallback_branch: str = None Default branch to fallback to

        :returns ApiResponse
        """
        url_path = f"/dbs/{db_name}/resolveBranch"
        query_params = []
        if git_branch is not None:
            query_params.append(f"gitBranch={git_branch}")
        if fallback_branch is not None:
            query_params.append(f"fallbackBranch={fallback_branch}")
        if query_params:
            url_path += "?" + "&".join(query_params)
        return self.request("GET", url_path)



#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# ------------------------------------------------------- #
# Migrations
# Branch schema migrations and history.
# Specification: workspace:v1.0
# ------------------------------------------------------- #

from xata.api_request import ApiRequest
from xata.api_response import ApiResponse


class Migrations(ApiRequest):

    scope = "workspace"

    def get_history(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Get branch migration history [deprecated]

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/migrations#get-branch-migration-history-[deprecated]
        Path: /db/{db_branch_name}/migrations
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/migrations"
        headers = {"content-type": "application/json"}
        return self.request("GET", url_path, headers, payload)

    def get_plan(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Compute a migration plan from a target schema the branch should be migrated too.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/migrations/plan#compute-migration-plan-[deprecated]
        Path: /db/{db_branch_name}/migrations/plan
        Method: POST
        Response status codes:
        - 200: Example response
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/migrations/plan"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def execute_plan(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Apply a migration plan to the branch

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/migrations/execute#migrate-branch-[deprecated]
        Path: /db/{db_branch_name}/migrations/execute
        Method: POST
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/migrations/execute"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def get_schema_history(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Query schema history.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/history#query-schema-history.
        Path: /db/{db_branch_name}/schema/history
        Method: POST
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/history"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def compare_branch_with_user_schema(
        self, payload: dict, db_name: str = None, branch_name: str = None
    ) -> ApiResponse:
        """
        Compare branch with user schema.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/compare#compare-branch-with-user-schema.
        Path: /db/{db_branch_name}/schema/compare
        Method: POST
        Response status codes:
        - 200: Schema comparison response.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/compare"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def compare_schemas(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Compare branch schemas.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/compare/branch_name#compare-branch-schemas.
        Path: /db/{db_branch_name}/schema/compare/{branch_name}
        Method: POST
        Response status codes:
        - 200: Schema comparison response.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/compare/{branch_name}"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def upadte_schema(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Update Branch schema

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/update#update-branch-schema
        Path: /db/{db_branch_name}/schema/update
        Method: POST
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/update"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def preview(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Preview branch schema edits.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/preview#preview-branch-schema-edits.
        Path: /db/{db_branch_name}/schema/preview
        Method: POST
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/preview"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def apply(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Apply edit script.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/apply#apply-edit-script.
        Path: /db/{db_branch_name}/schema/apply
        Method: POST
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/apply"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def push(self, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        The `schema/push` API accepts a list of migrations to be applied to the
        current branch. A list of applicable migrations can be fetched using
        the `schema/history` API from another branch or database.

        The most recent migration must be part of the list or referenced (via
        `parentID`) by the first migration in the list of migrations to be pushed.

        Each migration in the list has an `id`, `parentID`, and `checksum`. The
        checksum for migrations are generated and verified by xata. The
        operation fails if any migration in the list has an invalid checksum.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/schema/push#push-migrations.
        Path: /db/{db_branch_name}/schema/push
        Method: POST
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/schema/push"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)



#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

import logging

from requests import Session, request

from xata.api_response import ApiResponse

from .errors import RateLimitError, UnauthorizedError, XataServerError


class ApiRequest:
    def __init__(self, client):
        self.session = Session()
        self.client = client
        self.logger = logging.getLogger(self.__class__.__name__)

    def get_scope(self) -> str:
        return self.scope

    def is_control_plane(self) -> bool:
        return self.get_scope() == "core"

    def get_base_url(self) -> str:
        if self.is_control_plane():
            return "https://" + self.client.get_config()["domain_core"]
        # Base URL must be build on the fly as the region & workspace Id can change
        cfg = self.client.get_config()
        return "https://%s.%s.%s" % (cfg["workspaceId"], cfg["region"], cfg["domain_workspace"])

    def get_upload_base_url(self) -> str:
        """
        Upload of files has dedidcated URL that is not represented in the OAS

        :returns str
        """
        cfg = self.client.get_config()
        return "https://%s.%s.upload.%s" % (cfg["workspaceId"], cfg["region"], cfg["domain_workspace"])

    def request(
        self,
        http_method: str,
        url_path: str,
        headers: dict = {},
        payload: dict = None,
        data: bytes = None,
        is_streaming: bool = False,
        override_base_url=None,
    ) -> ApiResponse:
        """
        :param http_method: str
        :param url_path: str
        :headers: dict = {}
        :param payload: dict = None
        :param data: bytes = None
        :param is_streaming: bool = False
        :param override_base_url = None Set alternative base URL

        :returns ApiResponse

        :raises RateLimitError
        :raises UnauthorizedError
        :raises ServerError
        """
        headers = {**headers, **self.client.get_headers()}
        base_url = self.get_base_url() if override_base_url is None else override_base_url
        url = "%s/%s" % (base_url, url_path.lstrip("/"))

        # In order not exhaust the connection pool with open connections from unread streams
        # we opt for Session usage on all non-stream requests
        # https://requests.readthedocs.io/en/latest/user/advanced/#body-content-workflow
        if is_streaming:
            if payload is None and data is None:
                resp = request(http_method, url, headers=headers, stream=True)
            elif data is not None:
                resp = request(http_method, url, headers=headers, data=data, stream=True)
            else:
                resp = request(http_method, url, headers=headers, json=payload, stream=True)
        else:
            if payload is None and data is None:
                resp = self.session.request(http_method, url, headers=headers)
            elif data is not None:
                resp = self.session.request(http_method, url, headers=headers, data=data)
            else:
                resp = self.session.request(http_method, url, headers=headers, json=payload)

        # Any special status code we can raise an exception for ?
        if resp.status_code == 429:
            raise RateLimitError(f"code: {resp.status_code}, rate limited: {resp.json()}")
        if resp.status_code == 401:
            raise UnauthorizedError(f"code: {resp.status_code}, unauthorized: {resp.json()}")
        elif resp.status_code >= 500:
            raise XataServerError(f"code: {resp.status_code}, server error: {resp.text}")

        return ApiResponse(resp)



#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# ------------------------------------------------------- #
# Table
# Table management.
# Specification: workspace:v1.0
# ------------------------------------------------------- #

from xata.api_request import ApiRequest
from xata.api_response import ApiResponse


class Table(ApiRequest):

    scope = "workspace"

    def create(self, table_name: str, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Creates a new table with the given name. Returns 422 if a table with the same name already exists.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name#create-table
        Path: /db/{db_branch_name}/tables/{table_name}
        Method: PUT
        Response status codes:
        - 201: Created
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param table_name: str The Table name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}"
        return self.request("PUT", url_path)

    def delete(self, table_name: str, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Deletes the table with the given name.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name#delete-table
        Path: /db/{db_branch_name}/tables/{table_name}
        Method: DELETE
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Not Found
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param table_name: str The Table name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}"
        return self.request("DELETE", url_path)

    def update(self, table_name: str, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Update table. Currently there is only one update operation supported: renaming the table by providing a new name.

        In the example below, we rename a table from “users” to “people”:

        ```json
        // PATCH /db/test:main/tables/users

        {
          "name": "people"
        }
        ```

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name#update-table
        Path: /db/{db_branch_name}/tables/{table_name}
        Method: PATCH
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 422: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}"
        headers = {"content-type": "application/json"}
        return self.request("PATCH", url_path, headers, payload)

    def get_schema(self, table_name: str, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Get table schema

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/schema#get-table-schema
        Path: /db/{db_branch_name}/tables/{table_name}/schema
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param table_name: str The Table name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/schema"
        return self.request("GET", url_path)

    def set_schema(self, table_name: str, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Update table schema

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/schema#update-table-schema
        Path: /db/{db_branch_name}/tables/{table_name}/schema
        Method: PUT
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 409: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/schema"
        headers = {"content-type": "application/json"}
        return self.request("PUT", url_path, headers, payload)

    def get_columns(self, table_name: str, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Retrieves the list of table columns and their definition. This endpoint returns the column list with object columns being reported with their
        full dot-separated path (flattened).

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/columns#list-table-columns
        Path: /db/{db_branch_name}/tables/{table_name}/columns
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param table_name: str The Table name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/columns"
        return self.request("GET", url_path)

    def add_column(self, table_name: str, payload: dict, db_name: str = None, branch_name: str = None) -> ApiResponse:
        """
        Adds a new column to the table. The body of the request should contain the column definition.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/columns#create-new-column
        Path: /db/{db_branch_name}/tables/{table_name}/columns
        Method: POST
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/columns"
        headers = {"content-type": "application/json"}
        return self.request("POST", url_path, headers, payload)

    def get_column(
        self, table_name: str, column_name: str, db_name: str = None, branch_name: str = None
    ) -> ApiResponse:
        """
        Get the definition of a single column.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/columns/column_name#get-column-information
        Path: /db/{db_branch_name}/tables/{table_name}/columns/{column_name}
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error
        Response: application/json

        :param table_name: str The Table name
        :param column_name: str The Column name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/columns/{column_name}"
        return self.request("GET", url_path)

    def delete_column(
        self, table_name: str, column_name: str, db_name: str = None, branch_name: str = None
    ) -> ApiResponse:
        """
        Deletes the specified column.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/columns/column_name#delete-column
        Path: /db/{db_branch_name}/tables/{table_name}/columns/{column_name}
        Method: DELETE
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param column_name: str The Column name
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/columns/{column_name}"
        return self.request("DELETE", url_path)

    def update_column(
        self, table_name: str, column_name: str, payload: dict, db_name: str = None, branch_name: str = None
    ) -> ApiResponse:
        """
        Update column with partial data. Can be used for renaming the column by providing a new "name" field.

        Reference: https://xata.io/docs/api-reference/db/db_branch_name/tables/table_name/columns/column_name#update-column
        Path: /db/{db_branch_name}/tables/{table_name}/columns/{column_name}
        Method: PATCH
        Response status codes:
        - 200: Schema migration response with ID and migration status.
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        - default: Unexpected Error

        :param table_name: str The Table name
        :param column_name: str The Column name
        :param payload: dict content
        :param db_name: str = None The name of the database to query. Default: database name from the client.
        :param branch_name: str = None The name of the branch to query. Default: branch name from the client.

        :returns ApiResponse
        """
        db_branch_name = self.client.get_db_branch_name(db_name, branch_name)
        url_path = f"/db/{db_branch_name}/tables/{table_name}/columns/{column_name}"
        headers = {"content-type": "application/json"}
        return self.request("PATCH", url_path, headers, payload)



#
# Licensed to Xatabase, Inc under one or more contributor
# license agreements. See the NOTICE file distributed with
# this work for additional information regarding copyright
# ownership. Xatabase, Inc licenses this file to you under the
# Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You
# may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#

# ------------------------------------------------------- #
# Oauth
# OAuth
# Specification: core:v1.0
# ------------------------------------------------------- #

from xata.api_request import ApiRequest
from xata.api_response import ApiResponse


class Oauth(ApiRequest):

    scope = "core"

    def get_clients(self) -> ApiResponse:
        """
        Retrieve the list of OAuth Clients that a user has authorized

        Reference: https://xata.io/docs/api-reference/user/oauth/clients#get-the-list-of-user-oauth-clients
        Path: /user/oauth/clients
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        Response: application/json


        :returns ApiResponse
        """
        url_path = "/user/oauth/clients"
        return self.request("GET", url_path)

    def delete_clients(self, client_id: str) -> ApiResponse:
        """
        Delete the oauth client for the user and revoke all access

        Reference: https://xata.io/docs/api-reference/user/oauth/clients/client_id#delete-the-oauth-client-for-the-user
        Path: /user/oauth/clients/{client_id}
        Method: DELETE
        Response status codes:
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error

        :param client_id: str

        :returns ApiResponse
        """
        url_path = f"/user/oauth/clients/{client_id}"
        return self.request("DELETE", url_path)

    def get_access_tokens(self) -> ApiResponse:
        """
        Retrieve the list of valid OAuth Access Tokens on the current user's account

        Reference: https://xata.io/docs/api-reference/user/oauth/tokens#get-the-list-of-user-oauth-access-tokens
        Path: /user/oauth/tokens
        Method: GET
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 5XX: Unexpected Error
        Response: application/json


        :returns ApiResponse
        """
        url_path = "/user/oauth/tokens"
        return self.request("GET", url_path)

    def delete_access_tokens(self, token: str) -> ApiResponse:
        """
        Expires the access token for a third party app

        Reference: https://xata.io/docs/api-reference/user/oauth/tokens/token#delete-an-access-token-for-a-third-party-app
        Path: /user/oauth/tokens/{token}
        Method: DELETE
        Response status codes:
        - 204: No Content
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 409: Example response
        - 5XX: Unexpected Error

        :param token: str

        :returns ApiResponse
        """
        url_path = f"/user/oauth/tokens/{token}"
        return self.request("DELETE", url_path)

    def update_access_tokens(self, token: str, payload: dict) -> ApiResponse:
        """
        Updates partially the access token for a third party app

        Reference: https://xata.io/docs/api-reference/user/oauth/tokens/token#updates-an-access-token-for-a-third-party-app
        Path: /user/oauth/tokens/{token}
        Method: PATCH
        Response status codes:
        - 200: OK
        - 400: Bad Request
        - 401: Authentication Error
        - 404: Example response
        - 409: Example response
        - 5XX: Unexpected Error
        Response: application/json

        :param token: str
        :param payload: dict content

        :returns ApiResponse
        """
        url_path = f"/user/oauth/tokens/{token}"
        headers = {"content-type": "application/json"}
        return self.request("PATCH", url_path, headers, payload)
